@@ -1,26 +1,26 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-
-
-   make-iso.c: code to generate iso files
- 
-   Copyright (C) 2002-2004 Red Hat, Inc.
-  
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-  
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-  
-   You should have received a copy of the GNU General Public
-   License along with this program; if not, write to the
-   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
-  
-   Authors: Alexander Larsson <alexl@redhat.com>
-*/
+ * 
+ * make-iso.c: code to generate iso files
+ *
+ * Copyright (C) 2002-2004 Red Hat, Inc.
+ * 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Alexander Larsson <alexl@redhat.com>
+ */
 
 #include "config.h"
 
@@ -32,10 +32,8 @@
 #endif /* __FreeBSD__ || __NetBSD__ || __OpenBSD__ */
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef HAVE_STATVFS
-#include <sys/statvfs.h>
-#endif
 #include <time.h>
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -52,30 +50,58 @@
 #include "mkdtemp.h"
 #endif
 
+static gboolean
+make_iso_get_free_space (const char *filename, GnomeVFSFileSize *size)
+{
+	GnomeVFSURI *uri;
+
+	*size = 0;
+
+	uri = gnome_vfs_uri_new (filename);
+	if (uri == NULL)
+		return FALSE;
+
+	if (gnome_vfs_get_volume_free_space (uri, size) != GNOME_VFS_OK) {
+		gnome_vfs_uri_unref (uri);
+		return FALSE;
+	}
+
+	gnome_vfs_uri_unref (uri);
+	return TRUE;
+}
+
 struct mkisofs_output {
-	GMainLoop *loop;
-	int result;
-	int pid;
+	GMainLoop  *loop;
+	int         result;
+	int         pid;
 	const char *filename;
-	gboolean debug;
+	gboolean    debug;
 };
 
 static struct mkisofs_output *mkisofs_output_ptr;
 
+/**
+ * nautilus_burn_make_iso_cancel:
+ *
+ * Cancel current creation process
+ *
+ **/
 void
-make_iso_cancel (void)
+nautilus_burn_make_iso_cancel (void)
 {
 	if (mkisofs_output_ptr)
-	{
-		kill (mkisofs_output_ptr->pid, SIGINT);
-		unlink (mkisofs_output_ptr->filename);
+		{
+			kill (mkisofs_output_ptr->pid, SIGINT);
+			unlink (mkisofs_output_ptr->filename);
 
-		g_main_loop_quit (mkisofs_output_ptr->loop);
-	}
+			g_main_loop_quit (mkisofs_output_ptr->loop);
+		}
 }
 
 static void
-write_all (int fd, char *buf, int len)
+write_all (int   fd,
+	   char *buf,
+	   int   len)
 {
 	int bytes;
 	int res;
@@ -92,12 +118,13 @@ write_all (int fd, char *buf, int len)
 }
 
 static void
-copy_file (const char *source, const char *dest)
+copy_file (const char *source,
+	   const char *dest)
 {
-	int sfd, dfd;
+	int         sfd, dfd;
 	struct stat stat_buf;
-	char buffer[1024*8];
-	ssize_t len;
+	char        buffer [1024*8];
+	ssize_t     len;
 
 	if (link (source, dest) == 0) {
 		return;
@@ -132,9 +159,9 @@ copy_file (const char *source, const char *dest)
 static char *
 escape_path (const char *str)
 {
-	char *escaped, *d;
+	char       *escaped, *d;
 	const char *s;
-	int len;
+	int         len;
 	
 	s = str;
 	len = 1;
@@ -168,11 +195,11 @@ escape_path (const char *str)
 static gboolean
 dir_is_empty (const char *virtual_path)
 {
-	GnomeVFSFileInfo *info;
+	GnomeVFSFileInfo        *info;
 	GnomeVFSDirectoryHandle *handle;
-	GnomeVFSResult result;
-	char *escaped_path, *uri;
-	gboolean found_file;
+	GnomeVFSResult           result;
+	char                    *escaped_path, *uri;
+	gboolean                 found_file;
 	
 	escaped_path = gnome_vfs_escape_path_string (virtual_path);
 	uri = g_strconcat ("burn:///", escaped_path, NULL);
@@ -194,9 +221,9 @@ dir_is_empty (const char *virtual_path)
 			break;
 
 		/* Skip "." and "..".  */
-		if (info->name[0] == '.'
-		    && (info->name[1] == 0
-			|| (info->name[1] == '.' && info->name[2] == 0))) {
+		if (info->name [0] == '.'
+		    && (info->name [1] == 0
+			|| (info->name [1] == '.' && info->name [2] == 0))) {
 			gnome_vfs_file_info_clear (info);
 			continue;
 		}
@@ -215,9 +242,9 @@ static char *
 get_backing_file (const char *virtual_path)
 {
 	GnomeVFSHandle *handle;
-	GnomeVFSResult res;
-	char *escaped_path, *uri;
-	char *mapping;
+	GnomeVFSResult  res;
+	char           *escaped_path, *uri;
+	char           *mapping;
 
 	escaped_path = gnome_vfs_escape_path_string (virtual_path);
 	uri = g_strconcat ("burn:///", escaped_path, NULL);
@@ -242,13 +269,15 @@ static gboolean
 ask_disable_joliet (GtkWindow *parent)
 {
 	GtkWidget *dialog;
-	int res;
+	int        res;
 
 	dialog = gtk_message_dialog_new (parent,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_QUESTION,
 					 GTK_BUTTONS_OK_CANCEL,
-					 _("Some files don't have a suitable name for a Windows-compatible CD.\nDo you want to continue with Windows compatibility disabled?"));
+					 _("Disable Microsoft Windows compatibility?"));
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  _("Some files don't have a suitable name for a Windows-compatible CD.\nDo you want to continue with Windows compatibility disabled?"));
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Windows compatibility"));
 	res = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
@@ -256,13 +285,13 @@ ask_disable_joliet (GtkWindow *parent)
 }
 
 struct mkisofs_state {
-	FILE *graft_file;
-	char *emptydir;
-	int depth;
-	char *tmpdir;
-	char *copy_to_dir;
-	int copy_depth;
-	GList *remove_files;
+	FILE    *graft_file;
+	char    *emptydir;
+	int      depth;
+	char    *tmpdir;
+	char    *copy_to_dir;
+	int      copy_depth;
+	GList   *remove_files;
 	gboolean found_file;
 };
 
@@ -286,10 +315,10 @@ graft_file_end_dir_visitor (struct mkisofs_state *state)
 }
 
 static gboolean
-graft_file_visitor (const gchar *rel_path,
-		    GnomeVFSFileInfo *info,
+graft_file_visitor (const gchar          *rel_path,
+		    GnomeVFSFileInfo     *info,
 		    struct mkisofs_state *state,
-		    gboolean *recurse)
+		    gboolean             *recurse)
 {
 	char *mapping, *path1, *path2;
 	char *new_copy_dir;
@@ -362,14 +391,14 @@ graft_file_visitor (const gchar *rel_path,
 }
 	
 static void
-create_graft_file (GnomeVFSURI *uri,
-		   const gchar *prefix,
+create_graft_file (GnomeVFSURI          *uri,
+		   const gchar          *prefix,
 		   struct mkisofs_state *state)
 {
-	GnomeVFSFileInfo *info;
+	GnomeVFSFileInfo        *info;
 	GnomeVFSDirectoryHandle *handle;
-	GnomeVFSResult result;
-	gboolean stop;
+	GnomeVFSResult           result;
+	gboolean                 stop;
 
 	result = gnome_vfs_directory_open_from_uri (&handle, uri, GNOME_VFS_FILE_INFO_DEFAULT);
 	if (result != GNOME_VFS_OK)
@@ -387,9 +416,9 @@ create_graft_file (GnomeVFSURI *uri,
 			break;
 
 		/* Skip "." and "..".  */
-		if (info->name[0] == '.'
-		    && (info->name[1] == 0
-			|| (info->name[1] == '.' && info->name[2] == 0))) {
+		if (info->name [0] == '.'
+		    && (info->name [1] == 0
+			|| (info->name [1] == '.' && info->name [2] == 0))) {
 			gnome_vfs_file_info_clear (info);
 			continue;
 		}
@@ -468,13 +497,12 @@ mkisofs_stderr_read (GIOChannel   *source,
 		     gpointer      data)
 {
 	struct mkisofs_output *mkisofs_output = data;
-	char *line;
-	char fraction_str[7];
-	double fraction;
-	GIOStatus status;
+	char                  *line;
+	char                   fraction_str [7];
+	double                 fraction;
+	GIOStatus              status;
 
-	status = g_io_channel_read_line (source,
-					 &line, NULL, NULL, NULL);
+	status = g_io_channel_read_line (source, &line, NULL, NULL, NULL);
 
 	if (line && mkisofs_output->debug) {
 		g_print ("make_iso stderr: %s", line);
@@ -482,23 +510,23 @@ mkisofs_stderr_read (GIOChannel   *source,
 
 	if (status == G_IO_STATUS_NORMAL) {
 		if (strncmp (line, "Total translation table size", 28) == 0) {
-			cd_progress_set_fraction (1.0);
+			nautilus_burn_progress_set_fraction (1.0);
 			g_main_loop_quit (mkisofs_output->loop);
-			mkisofs_output->result = RESULT_FINISHED;
+			mkisofs_output->result = NAUTILUS_BURN_RECORDER_RESULT_FINISHED;
 		}
 
 		if (strstr (line, "estimate finish")) {
 			if (sscanf (line, "%6c%% done, estimate finish",
-						fraction_str) == 1) {
-				fraction_str[6] = 0;
+				    fraction_str) == 1) {
+				fraction_str [6] = 0;
 				fraction = g_strtod (fraction_str, NULL);
-				cd_progress_set_fraction (fraction/100.0);
+				nautilus_burn_progress_set_fraction (fraction/100.0);
 			}
 		}
 		if (strstr (line, "Incorrectly encoded string")) {
-			cd_progress_set_fraction (1.0);
+			nautilus_burn_progress_set_fraction (1.0);
 			g_main_loop_quit (mkisofs_output->loop);
-			mkisofs_output->result = RESULT_ERROR;
+			mkisofs_output->result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 		}
 
 		g_free (line);
@@ -507,37 +535,45 @@ mkisofs_stderr_read (GIOChannel   *source,
 }
 
 /**
+ * nautilus_burn_make_iso:
+ * @filename:
+ * @label:
+ * @warn_low_space:
+ * @use_joliet:
+ * @debug:
+ *
  * Create an ISO image in filename from the data files in burn:///
- */
+ *
+ * Return value: 
+ **/
 int
-make_iso (const char *filename, const char *label,
-		gboolean warn_low_space, gboolean use_joliet, gboolean debug)
+nautilus_burn_make_iso (const char *filename,
+			const char *label,
+			gboolean    warn_low_space,
+			gboolean    use_joliet,
+			gboolean    debug)
 {
-	GnomeVFSURI *uri;
-	char *filelist = NULL;
-	struct mkisofs_state state = {NULL};
-	char *tempdir;
-	GList *l;
-	int stdout_pipe, stderr_pipe;
-	int i;
-	GError *error;
-	const char *argv[20]; /* Shouldn't need more than 20 arguments */
+	GnomeVFSURI          *uri;
+	char                 *filelist = NULL;
+	struct mkisofs_state  state = {NULL};
+	char                 *tempdir;
+	GList                *l;
+	int                   stdout_pipe, stderr_pipe;
+	int                   i;
+	GError               *error;
+	const char           *argv [20]; /* Shouldn't need more than 20 arguments */
 	struct mkisofs_output mkisofs_output;
-	GIOChannel *channel;
-	guint stdout_tag, stderr_tag;
-	char *stdout_data, *stderr_data;
-	char *dirname;
-	int exit_status, res;
-	unsigned long iso_size;
-#ifdef HAVE_STATVFS
-	struct statvfs statfs_buf;
-#else
-	struct statfs statfs_buf;
-#endif
-	GtkWidget *dialog;
+	GIOChannel           *channel;
+	guint                 stdout_tag, stderr_tag;
+	char                 *stdout_data, *stderr_data;
+	char                 *dirname;
+	int                   exit_status, res;
+	unsigned long         iso_size;
+	GtkWidget            *dialog;
+	GnomeVFSFileSize      size;
 
 	if (label) {
-		g_return_val_if_fail (strlen (label) < 32, RESULT_ERROR);
+		g_return_val_if_fail (strlen (label) < 32, NAUTILUS_BURN_RECORDER_RESULT_ERROR);
 	}
 
 	mkisofs_output.debug = debug;
@@ -547,7 +583,7 @@ make_iso (const char *filename, const char *label,
 	g_free (dirname);
 	state.tmpdir = mkdtemp (tempdir);
 
-	mkisofs_output.result = RESULT_ERROR;
+	mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 
 	if (state.tmpdir == 0) {
 		g_warning ("Unable to create temp dir\n");
@@ -574,42 +610,44 @@ make_iso (const char *filename, const char *label,
 	state.remove_files = g_list_prepend (state.remove_files, g_strdup (filelist));
 
 	if (!state.found_file) {
-		dialog = gtk_message_dialog_new (cd_progress_get_window (),
+		dialog = gtk_message_dialog_new (nautilus_burn_progress_get_window (),
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_CLOSE,
-						 _("No files selected to write to CD."));
+						 _("No files selected"));
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+							  _("There are no files to write to disc."));
 		gtk_window_set_title (GTK_WINDOW (dialog),
 				      _("No files selected"));
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
-		mkisofs_output.result = RESULT_ERROR;
+		mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 		goto cleanup;
 	}
 
-retry:
+ retry:
 	i = 0;
-	argv[i++] = "mkisofs";
-	argv[i++] = "-r";
+	argv [i++] = "mkisofs";
+	argv [i++] = "-r";
 	if (use_joliet) {
-		argv[i++] = "-J";
+		argv [i++] = "-J";
 	}
 	/* Undocumented -input-charset option */
-	argv[i++] = "-input-charset";
-	argv[i++] = "utf8";
-	argv[i++] = "-q";
-	argv[i++] = "-graft-points";
-	argv[i++] = "-path-list";
-	argv[i++] = filelist;
-	argv[i++] = "-print-size";
-	argv[i++] = NULL;
+	argv [i++] = "-input-charset";
+	argv [i++] = "utf8";
+	argv [i++] = "-q";
+	argv [i++] = "-graft-points";
+	argv [i++] = "-path-list";
+	argv [i++] = filelist;
+	argv [i++] = "-print-size";
+	argv [i++] = NULL;
 
 	error = NULL;
 
 	if (debug) {
 		g_print ("launching command: ");
-		for (i = 0; argv[i] != NULL; i++) {
-			g_print ("%s ", argv[i]);
+		for (i = 0; argv [i] != NULL; i++) {
+			g_print ("%s ", argv [i]);
 		}
 		g_print ("\n");
 	}
@@ -626,7 +664,7 @@ retry:
 		g_warning ("mkisofs command failed: %s\n", error->message);
 		g_error_free (error);
 		/* TODO: Better error handling */
-		mkisofs_output.result = RESULT_ERROR;
+		mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 		goto cleanup;
 	}
 
@@ -634,11 +672,11 @@ retry:
 		if (strstr (stderr_data, "Joliet tree sort failed.") != NULL) {
 			g_free (stdout_data);
 			g_free (stderr_data);
-			if (ask_disable_joliet (cd_progress_get_window ())) {
+			if (ask_disable_joliet (nautilus_burn_progress_get_window ())) {
 				use_joliet = FALSE;
 				goto retry;
 			} else {
-				mkisofs_output.result = RESULT_ERROR;
+				mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 				goto cleanup;
 			}
 		}
@@ -649,67 +687,66 @@ retry:
 	g_free (stdout_data);
 
 	dirname = g_path_get_dirname (filename);
-#ifdef HAVE_STATVFS
-	res = statvfs (dirname, &statfs_buf);
-#else
-	res = statfs (dirname, &statfs_buf);
-#endif
+	res = make_iso_get_free_space (dirname, &size);
+
 	if (res == -1) {
 		g_warning ("Cannot get free space at %s\n", dirname);
 		g_free (dirname);
-	} else if (iso_size / statfs_buf.f_bsize >= statfs_buf.f_bavail / 2048) {
+	} else if (iso_size > size) {
 		g_free (dirname);
 		if (warn_low_space) {
-			dialog = gtk_message_dialog_new (cd_progress_get_window (),
+			dialog = gtk_message_dialog_new (nautilus_burn_progress_get_window (),
 							 GTK_DIALOG_DESTROY_WITH_PARENT,
 							 GTK_MESSAGE_ERROR,
 							 GTK_BUTTONS_CLOSE,
-							 ngettext("Not enough space to store CD image (%ld Megabyte needed)",
-								  "Not enough space to store CD image (%ld Megabytes needed)",
-								  iso_size / 512),
-							 iso_size / 512);
+							 _("Not enough space"));
+			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+								  ngettext("Not enough space to store disc image (%ld Megabyte needed)",
+									   "Not enough space to store disc image (%ld Megabytes needed)",
+									   iso_size / 512),
+								  iso_size / 512);
 			gtk_window_set_title (GTK_WINDOW (dialog),
 					      _("Not enough space"));
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
-			mkisofs_output.result = RESULT_ERROR;
+			mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 		} else {
-			mkisofs_output.result = RESULT_RETRY;
+			mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_RETRY;
 		}
 
 		goto cleanup;
 	}
 
 	i = 0;
-	argv[i++] = "mkisofs";
-	argv[i++] = "-r";
+	argv [i++] = "mkisofs";
+	argv [i++] = "-r";
 	if (use_joliet) {
-		argv[i++] = "-J";
+		argv [i++] = "-J";
 	}
-	argv[i++] = "-input-charset";
-	argv[i++] = "utf8";
-	argv[i++] = "-graft-points";
-	argv[i++] = "-path-list";
-	argv[i++] = filelist;
+	argv [i++] = "-input-charset";
+	argv [i++] = "utf8";
+	argv [i++] = "-graft-points";
+	argv [i++] = "-path-list";
+	argv [i++] = filelist;
 	if (label) {
-		argv[i++] = "-V";
-		argv[i++] = label;
+		argv [i++] = "-V";
+		argv [i++] = label;
 	}
-	argv[i++] = "-o";
-	argv[i++] = filename;
-	argv[i++] = NULL;
+	argv [i++] = "-o";
+	argv [i++] = filename;
+	argv [i++] = NULL;
 
 	if (debug) {
 		g_print ("launching command: ");
-		for (i = 0; argv[i] != NULL; i++) {
-			g_print ("%s ", argv[i]);
+		for (i = 0; argv [i] != NULL; i++) {
+			g_print ("%s ", argv [i]);
 		}
 		g_print ("\n");
 	}
 
-	cd_progress_set_text (_("Creating CD image"));
-	cd_progress_set_fraction (0.0);
-	cd_progress_set_image_spinning (TRUE);
+	nautilus_burn_progress_set_text (_("Creating CD image"));
+	nautilus_burn_progress_set_fraction (0.0);
+	nautilus_burn_progress_set_image_spinning (TRUE);
 	error = NULL;
 	if (!g_spawn_async_with_pipes  (NULL,
 					(char **)argv,
@@ -723,7 +760,7 @@ retry:
 					&error)) {
 		g_warning ("mkisofs command failed: %s\n", error->message);
 		g_error_free (error);
-		mkisofs_output.result = RESULT_ERROR;
+		mkisofs_output.result = NAUTILUS_BURN_RECORDER_RESULT_ERROR;
 		goto cleanup;
 	} else {
 		mkisofs_output.loop = g_main_loop_new (NULL, FALSE);
@@ -765,8 +802,132 @@ retry:
 	g_free (state.emptydir);
 	rmdir (tempdir);
 	g_free (tempdir);
-	cd_progress_set_image_spinning (FALSE);
+	nautilus_burn_progress_set_image_spinning (FALSE);
 
 	return mkisofs_output.result;
 }
 
+/* Originally eel_make_valid_utf8 */
+static char *
+ncb_make_valid_utf8 (const char *name)
+{
+	GString    *string;
+	const char *remainder, *invalid;
+	int         remaining_bytes, valid_bytes;
+
+	string = NULL;
+	remainder = name;
+	remaining_bytes = strlen (name);
+
+	while (remaining_bytes != 0) {
+		if (g_utf8_validate (remainder, remaining_bytes, &invalid)) {
+			break;
+		}
+		valid_bytes = invalid - remainder;
+
+		if (string == NULL) {
+			string = g_string_sized_new (remaining_bytes);
+		}
+		g_string_append_len (string, remainder, valid_bytes);
+		g_string_append_c (string, '?');
+
+		remaining_bytes -= valid_bytes + 1;
+		remainder = invalid + 1;
+	}
+
+	if (string == NULL) {
+		return g_strdup (name);
+	}
+
+	g_string_append (string, remainder);
+	g_string_append (string, _(" (invalid Unicode)"));
+	g_assert (g_utf8_validate (string->str, -1, NULL));
+
+	return g_string_free (string, FALSE);
+}
+
+/**
+ * nautilus_burn_verify_iso:
+ * @filename:
+ * @iso_label:
+ * @error:
+ *
+ * Verify that filename is a valid ISO image
+ *
+ * Return value: %TRUE if filename is a valid ISO image, otherwise %FALSE
+ **/
+gboolean
+nautilus_burn_verify_iso (const char *filename,
+			  char      **iso_label,
+			  GError    **error)
+{
+	FILE  *file;
+#define BUFFER_SIZE 128
+	char  buf [BUFFER_SIZE+1];
+	int   res;
+	char *str, *str2;
+
+	file = fopen (filename, "rb");
+	if (file == NULL) {
+		int err = errno;
+		*error = g_error_new_literal (g_file_error_quark (),
+					      g_file_error_from_errno (err),
+					      strerror (err));
+		return FALSE;
+	}
+	/* Verify we have an ISO image */
+	/* This check is for the raw sector images */
+	res = fseek (file, 37633L, SEEK_SET);
+	if (res) {
+		goto bail;
+	}
+	res = fread (buf, sizeof (char), 5, file);
+	if (res != 5 || strncmp (buf, "CD001", 5) != 0) {
+		/* Standard ISO images */
+		res = fseek (file, 32769L, SEEK_SET);
+		if (res) {
+			goto bail;
+		}
+		res = fread (buf, sizeof (char), 5, file);
+		if (res != 5 || strncmp (buf, "CD001", 5) != 0) {
+			/* High Sierra images */
+			res = fseek (file, 32776L, SEEK_SET);
+			if (res) {
+				goto bail;
+			}
+			res = fread (buf, sizeof (char), 5, file);
+			if (res != 5 || strncmp (buf, "CDROM", 5) != 0) {
+				goto bail;
+			}
+		}
+	}
+	/* Extract the volume label from the image */
+	res = fseek (file, 32808L, SEEK_SET);
+	if (res) {
+		goto bail;
+	}
+	res = fread (buf, sizeof(char), BUFFER_SIZE, file);
+	if (res != BUFFER_SIZE) {
+		goto bail;
+	}
+	buf [BUFFER_SIZE] = '\0';
+	str = g_strdup (g_strstrip (buf));
+	if (!g_utf8_validate (str, -1, NULL)) {
+		/* Hmm, not UTF-8. Try the current locale. */
+		str2 = g_locale_to_utf8 (str, -1, NULL, NULL, NULL);
+		if (str2 == NULL) {
+			str2 = ncb_make_valid_utf8 (str);
+		}
+		g_free (str);
+		str = str2;
+	}
+	fclose (file);
+	*iso_label = str;
+	return TRUE;
+
+ bail:
+	*error = g_error_new_literal (0, 0, _("Not a valid CD image."));
+
+	return FALSE;
+}
+
